﻿


;-------------------------------------------------------------------------------
MultiWordAnagrams(String, SpaceCount) { ; return an array with the "sentences"
;-------------------------------------------------------------------------------
    Result := [], Previous := []
    Keyword := make_Keyword(String)

    If (SpaceCount = 1)
        Return, TwoWordAnagrams(Keyword)

    ; otherwise: get word pairs, iterate over requested size for Pair[1]
    Loop, % StrLen(Keyword) - A_Index {
        For each, Pair in get_Pairs(Keyword, A_Index) {
            For each, MultiWord in MultiWordAnagrams(Pair[2], SpaceCount - 1) {
                Candidate := Pair[1] " " MultiWord
                Sort, Candidate, D%A_Space%
                If Not Previous.HasKey(Candidate) {
                    Previous[Candidate] := True
                    Result.Push(Pair[1] " " MultiWord)
                }
            }
        }
    }

    Return, Result
}



;-------------------------------------------------------------------------------
get_Pairs(Keyword, n) { ; return a collection of [Key, Sibling]
;-------------------------------------------------------------------------------
    global DICT

    Result := []
    SIZE := StrLen(Keyword)

    For each, KeyList in DICT[n] {
        For each, Key in KeyList {
            Sibling := Complement(Keyword, Key)
            If StrLen(Key) + StrLen(Sibling) = SIZE ; valid Key
                Result.Push([Key, Sibling])
        }
    }

    Return, Result
}



;-------------------------------------------------------------------------------
TwoWordAnagrams(Keyword) { ; return an array with the "sentences"
;-------------------------------------------------------------------------------
    global Max_Input_Length, DICT

    Result := [], Previous := []
    Solutions := Combinations(Keyword, Max_Input_Length)
    Len0 := StrLen(Keyword)  ; # of letters in Keyword (Input without spaces)
    Len1 := (Len0 + 1) // 2  ; # of letters in longer word

    Loop, % Len0 - Len1 {
        Index1 := Len1 + A_Index - 1 ; iterate e.g. from 5 to 8 (for 9 letters)
        For Key, Size in Solutions[Index1] {
            Compl := Complement(Keyword, Key)
            If Not Previous.HasKey(Compl)
            And DICT[Size].HasKey(Key)
            And DICT[Len0 - Size].HasKey(Compl) {
                Previous[Key] := True
                For each, Anagram1 in DICT[StrLen(Key), Key]
                    For each, Anagram2 in DICT[StrLen(Compl), Compl]
                        Result.Push(Anagram1 " " Anagram2)
            }
        }
    }

    Return, Result
}



;-------------------------------------------------------------------------------
Complement(Keyword, Subkey) { ; remove from Keyword all the letters from Subkey
;-------------------------------------------------------------------------------
    Loop, Parse, Subkey
        Keyword := StrReplace(Keyword, A_LoopField,,, 1)
    Return, Keyword
}



;-------------------------------------------------------------------------------
Permutations(String) { ; return an array with all word permutations of String
;-------------------------------------------------------------------------------
    Words := StrSplit(String, A_Space)

    If (Len := Words.Length()) = 1
        Return, [String]

    Result := []
    Loop, % Len {
        Index := A_Index
        Split1 := Split3 := ""

        Loop, % Index - 1
            Split1 .= Words[A_Index] A_Space ; before pos

        Split2 := Words[Index] A_Space ; at pos

        Loop, % Len - Index
            Split3 .= Words[Index + A_Index] A_Space ; after pos

        For each, Perm in Permutations(RTrim(Split1 Split3))
            Result.Push(Split2 Perm)
    }

    Return, Result
}



;-------------------------------------------------------------------------------
Combinations(String, Skip) { ; return an array with the sub-keywords
;-------------------------------------------------------------------------------
    ; Sub-keywords are the keywords of all the sub-anagrams of String.
    ; The returned array contains sub-arrays, similar to DICT,
    ; where each sub-array is a collection of same-length keywords.
    ;---------------------------------------------------------------------------
    global Max_Input_Length

    Result := []
    Len := StrLen(String)
    Keyword := make_Keyword(String)
    Result[Len, Keyword] := Len

    Loop, % Len - 1 - (Skip = Max_Input_Length ? 0 : Skip) {
        n := Len - A_Index ; key for next "shorter" array
        For ShortKey in Result[n + 1] {
            Loop, % n + 1 {

                ; split the ShortKey and get next shorter keyword
                Split1 := SubStr(ShortKey, 1, A_Index - 1)  ; before pos
                Split3 := SubStr(ShortKey, A_Index + 1)     ; after pos

                If Not Result[n].HasKey(NextShorter := Split1 Split3)
                    Result[n, NextShorter] := n
            }
        }
    }

    Return, Result
}



;-------------------------------------------------------------------------------
LengthSort(a, b) { ; word length specific custom sort
;-------------------------------------------------------------------------------
    ; Compacting this function to a one-liner, similar to the examples
    ; in the help file, runs about 20% faster than before.
    ;---------------------------------------------------------------------------

    ; primary criterion is string length, otherwise sort alphabetically
    Return, StrLen(a) < StrLen(b) ? 1 : StrLen(a) > StrLen(b) ? -1
        : a > b ? 1 : a < b ? -1 : 0
}



;-------------------------------------------------------------------------------
CleanUp() { ; called by ExitApp, Reload and #SingleInstance
;-------------------------------------------------------------------------------
    global ; needed

    ; get LASTxxx info
    Gui, Main: Submit
    LastInput := Input
    LastSkip := Skip

    ControlGet, Column, CurrentCol,,, ahk_id %hEdit%
    LastPos := Column - 1

    ; save
    Gosub, Write_INI
}
