﻿;-------------------------------------------------------------------------------
; Spinner.ahk
; by wolf_II
;-------------------------------------------------------------------------------
; creates a spinning animattion
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Spinner(DotCount := 9, Diameter := 12, bClockwise := False) { ; create spinner
;-------------------------------------------------------------------------------
    ; DotCount      the number of dots to draw
    ; Diameter      the size of the dots to draw
    ; bClockwise    direction of the spinner
    ;---------------------------------------------------------------------------
    global

    gDotCount := DotCount
    gDiameter := Diameter
    gClockwise := bClockwise

    ; check first argument
    If (DotCount = "Off") {
        SetTimer, Spinner, Off
        SelectObject(hDC, oBM)
        DeleteObject(hBM)
        DeleteDC(hDC)
        Gdip_DeleteGraphics(pGraphics)
        Loop, %DotCount%
            Gdip_DeleteBrush(BRUSHES[A_Index])
        Gdip_Shutdown(pToken)
        Gui, Spinner: Destroy
        Return, False
    }

    ; setup
    Pi := 4 * ATan(1)                   ; trigonometry
    MidX := A_ScreenWidth // 2          ; x-coord of the center of the dots
    MidY := A_ScreenHeight // 2         ; y-coord of the center of the dots
    Radius := 20                        ; distance of the dots from the center

    ; setup Gdip
    pToken := Gdip_Startup()
    hDC := CreateCompatibleDC()
    hBM := CreateDIBSection(A_ScreenWidth, A_ScreenHeight)
    oBM := SelectObject(hDC, hBM)
    pGraphics := Gdip_GraphicsFromHDC(hDC)

    ; setup GUI
    Gui, Spinner: New, +AlwaysOnTop +hWndhGui +ToolWindow
    Gui, +E0x80020 ; WS_EX_LAYERED | WS_EX_TRANSPARENT (click through)
    Gui, Show, NA

    ; do all the maths first
    DOT := [], ARGB := [], BRUSHES := []
    Loop, %DotCount% {
        DOT[A_Index, "x"] := MidX + Radius * Cos(A_Index * 2 * Pi / DotCount)
        DOT[A_Index, "y"] := MidY + Radius * Sin(A_Index * 2 * Pi / DotCount)
        ARGB.Push(Format("0x{:x}", A_Index * 255 // DotCount) "000000")
        BRUSHES.Push(Gdip_BrushCreateSolid(ARGB[A_Index]))
    }

    ; start spinning
    Index := 0 ; initialize
    SetTimer, Spinner
    Return, True
}



ExitApp



;-------------------------------------------------------------------------------
Spinner: ; "animate" the GUI
;-------------------------------------------------------------------------------
    If (Index++ > 2 * gDotCount)
        Index -= gDotCount

    Gdip_GraphicsClear(pGraphics)
    Loop, %gDotCount%
        Gdip_FillEllipse(pGraphics
            , BRUSHES[Mod(Index + A_Index * (gClockwise ? -1 : 1), gDotCount) + 1]
            , DOT[A_Index].x, DOT[A_Index].y, gDiameter, gDiameter)
    UpdateLayeredWindow(hGui, hDC, 0, 0, A_ScreenWidth, A_ScreenHeight)

Return



#Include, <Gdip>
